<?php

namespace App\Http\Controllers;

use App\Helper\Helper;
use Illuminate\Http\Request;
use App\Jobs\IterableSubscriptionCreateEvent;
use App\Jobs\IterableSubscriptionCancelEvent;
use App\Jobs\IterableSubscriptionChargeUpcomingEvent;
use App\Jobs\IterableUserChannelUpdateEvent;
use App\Jobs\IterableOnetimeCreateEvent;
use Illuminate\Support\Facades\DB;

class IterableWebhookController extends Controller
{

    private $iterableObj;
    private $checkObj;
    private $rechargeObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $shopifyWebhookSecret;
    private $rechargeWebhookSecret;

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public function __construct()
    {
        $this->shopifyWebhookSecret = env('SHOPIFY_WEBHOOK_SECRET', true);
        $this->rechargeWebhookSecret = env('RECHARGE_API_CLIENT_SECRET',true);
        $this->helperObj = new Helper();

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();

    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');
        if (!is_dir($this->logPath)){

         //Directory does not exist, so lets create it.
         mkdir($this->logPath);
         chmod($this->logPath, 0777);

        }
        $this->logFiles = array(
            'log' => array(
                "type" => "text",
                "filename" => $this->logPath . "/webhook-log.txt",
                "mode" => ""),
            );
    }

    public function setWithLogFiles() {

        if(!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                   $handler = fopen($fileName, $fileDetails['mode']);
                   fputcsv($handler, $fileDetails['columns']);
                   fclose($handler);
                   touch($fileName);
                   chmod($fileName, 0777);
                }
            }

        }
   }

    /**
     * Create Iterable event for created subscription
     */
    public function subscriptionCreatedEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);

        if ($verified) {

            $receivedData = json_decode($requestData);
            if(empty($receivedData->subscription->id)) {

                $this->helperObj->logMe("Error in the webhook response(Created subscription) -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);
            } else {

                $queueResponse = dispatch(new IterableSubscriptionCreateEvent(
                    $receivedData, $this->logFiles['log']['filename']));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            $this->helperObj->logMe('Recharge subscription create Webhook validation failed',
            $this->logFiles['log']['filename']);
        }
    }

    /**
     * Create Iterable event for cancelled subscription
     */

    public function subscriptionCanceledEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);

        if ($verified) {

            $receivedData = json_decode($requestData);

            if(empty($receivedData->subscription->id)) {

                $this->helperObj->logMe("Error in the webhook response(Cancelled subscription) -
                Missing required parameters in the webhook response or the response is not correct",
                $this->logFiles['log']['filename']);
            } else {

                $queueResponse = dispatch(new IterableSubscriptionCancelEvent(
                    $receivedData->subscription->customer_id, $this->logFiles['log']['filename']));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            $this->helperObj->logMe('Recharge subscription cancelled Webhook validation failed',
            $this->logFiles['log']['filename']);
        }

     }

      /**
     * Create Iterable event for upcoming charges
     */

    public function upcomingChargesEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);

        if ($verified) {

            $receivedData = json_decode($requestData, true);

            if(empty($receivedData['charge']['id'])) {

                $this->helperObj->logMe("Error in the webhook response(upcoming charge) -
                Missing required parameters in the webhook response or the response is not correct",
                $this->logFiles['log']['filename']);
            } else {

                $queueResponse = dispatch(new IterableSubscriptionChargeUpcomingEvent(
                  $receivedData, $this->logFiles['log']['filename']));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            $this->helperObj->logMe('Recharge upcoming charge Webhook validation failed',
            $this->logFiles['log']['filename']);
        }

     }

     /**
     * Update user Iterable channel status
     */

    public function updateIterableUserChannelStatus() {

        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_shopify_webhook($requestData, $hmac_header);
        $webhookStatus = var_export($verified, true);

        if ($webhookStatus) {

            $receivedData = json_decode($requestData);

            if(empty($receivedData->email)) {

                $this->helperObj->logMe("Error in the iterable user channel update -
                Missing required parameters in the webhook response or the response is not correct",
                $this->logFiles['log']['filename']);
            } else {
                $queueResponse = dispatch(new IterableUserChannelUpdateEvent(
                    $receivedData->email, $this->logFiles['log']['filename']));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            $this->helperObj->logMe('Shopify customer creation webhook validation failed',$this->logFiles['log']['filename']);
        }
    }
    
    /**
     * Create Iterable event for created onetime
     */
    /*public function onetimeCreatedEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);
        
        $this->helperObj->logMe("Reached to onetime purchase", $this->logFiles['log']['filename']);

        if ($verified) {
            
            $this->helperObj->logMe($requestData, $this->logFiles['log']['filename']);

            $receivedData = json_decode($requestData);
            
                
            if(empty($receivedData->onetime->id)) {

                $this->helperObj->logMe("Error in the webhook response(Created OTP) -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);
            } else {

                $queueResponse = dispatch(new IterableOnetimeCreateEvent(
                    $receivedData, $this->logFiles['log']['filename']));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            $this->helperObj->logMe('Recharge onetime create Webhook validation failed',
            $this->logFiles['log']['filename']);
        }
    }*/
    
    public function orderCreateEventWebhook() {
        $this->helperObj->logMe("Reached to order creation webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);
        if ($verified) {
            
            $this->helperObj->logMe($requestData, $this->logFiles['log']['filename']);

            $receivedData = json_decode($requestData);
            
                
            if(empty($receivedData->order->id)) {

                $this->helperObj->logMe("Error in the webhook response(Created OTP) -
                Missing required parameters in the webhook response or the
                response is not correct", $this->logFiles['log']['filename']);
            } else {
                
                if($receivedData->order->tags){
                    $tags = explode(',', $receivedData->order->tags);
                    if(!empty($tags) && in_array('OTP', $tags)){
                        
                        $this->helperObj->logMe("OTP order received => ".$receivedData->order->id, $this->logFiles['log']['filename']);
                        
                    	$queueResponse = dispatch(new IterableOnetimeCreateEvent($receivedData, $this->logFiles['log']['filename']));
                		ignore_user_abort(true);
                		header("Content-Length: 0");
                		header("Connection: Close");
                		flush();
                    }else{
                        $this->helperObj->logMe("Subscription order received => ".$receivedData->order->id, $this->logFiles['log']['filename']);
                    }
                    
                }

                
            }
        } else {

            $this->helperObj->logMe('Recharge onetime create Webhook validation failed',
            $this->logFiles['log']['filename']);
        }
    }
    
    
    public function checkoutUpdatedWebhook() {
        $this->helperObj->logMe("----------------Reached to checkout updated webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);
        if ($verified) {

            $this->helperObj->logMe($requestData, $this->logFiles['log']['filename']);

            $receivedData = json_decode($requestData);

            if(!isset($receivedData->checkout->token)) {

                $this->helperObj->logMe("Checkout token not found in response", $this->logFiles['log']['filename']);
            } else {

                if(isset($receivedData->checkout->token) && $receivedData->checkout->completed_at == ''){
                    $updateValues = array(
                        'checkout_updated_at'=>$receivedData->checkout->updated_at,
                        'checkout_details' =>  json_encode($receivedData),    
                        'checkout_status'=>'updated'
                    );
                    $updateCheckout = DB::table('user_checkouts')
                    ->where(['user_email' => $receivedData->checkout->email])
                    ->where(['checkout_recharge_token' => $receivedData->checkout->token])
                    ->update($updateValues);


                    if($updateCheckout){
                        $this->helperObj->logMe("Checkout updated for user email => "
                        .$receivedData->checkout->email." and token =>".$receivedData->checkout->token,
                        $this->logFiles['log']['filename']);
                    }else{
                        $this->helperObj->logMe("Failed to update the Checkout for user email => "
                        .$receivedData->checkout->email." and token =>".$receivedData->checkout->token,
                        $this->logFiles['log']['filename']);
                    }
                }
                if(isset($receivedData->checkout->token) && $receivedData->checkout->completed_at != ''){
                    $updateValues = array(
                        'checkout_updated_at'=>$receivedData->checkout->updated_at,
                        'checkout_status'=>'completed'
                    );
                    $updateCheckout = DB::table('user_checkouts')
                    ->where(['user_email' => $receivedData->checkout->email])
                    ->where(['checkout_recharge_token' => $receivedData->checkout->token])
                    ->update($updateValues);


                    if($updateCheckout){
                        $this->helperObj->logMe("Checkout completed for user email => "
                        .$receivedData->checkout->email." and token =>".$receivedData->checkout->token,
                        $this->logFiles['log']['filename']);
                    }else{
                        $this->helperObj->logMe("Failed to completed the Checkout for user email => "
                        .$receivedData->checkout->email." and token =>".$receivedData->checkout->token,
                        $this->logFiles['log']['filename']);
                    }
                }

            }
        }else {

            $this->helperObj->logMe('Recharge checkoutUpdatedWebhook Webhook validation failed',
            $this->logFiles['log']['filename']);
        }
    }
    
    public function checkoutCreatedWebhook() {
        $this->helperObj->logMe("----------------Reached to checkout created webhook", $this->logFiles['log']['filename']);
        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);
        if ($verified) {

            $this->helperObj->logMe($requestData, $this->logFiles['log']['filename']);

            $receivedData = json_decode($requestData);

            if(!isset($receivedData->checkout->token)) {

                $this->helperObj->logMe("Checkout token not found in response", $this->logFiles['log']['filename']);
            } else {

                if(isset($receivedData->checkout->token) && $receivedData->checkout->completed_at == ''){
                    
                    
                    $lastInsertedId = DB::table('user_checkouts')->insertGetId(
                                        [
                                            'user_email' => $receivedData->checkout->email,
                                            'checkout_recharge_token' => $receivedData->checkout->token,
                            	            'checkout_created_at' => $receivedData->checkout->created_at,
                                            'checkout_updated_at' => $receivedData->checkout->created_at,
                                            'checkout_status' => 'created',
                                            'checkout_details' =>  json_encode($receivedData)                    
                                        ]
                                    );

                    if($lastInsertedId){
                        $this->helperObj->logMe("Checkout insert for user email => "
                        .$receivedData->checkout->email." and token =>".$receivedData->checkout->token,
                        $this->logFiles['log']['filename']);
                    }else{
                        $this->helperObj->logMe("Failed to insert the Checkout details for user email => "
                        .$receivedData->checkout->email." and token =>".$receivedData->checkout->token,
                        $this->logFiles['log']['filename']);
                    }
                }
            }
        }else {

            $this->helperObj->logMe('Recharge checkoutUpdatedWebhook Webhook validation failed',
            $this->logFiles['log']['filename']);
        }
    }
    

    public function verify_shopify_webhook($data, $hmac_header) {

        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->shopifyWebhookSecret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    }

    public function verify_recharge_webhook($data, $received_digest) {

        $calculated_digest = hash('sha256',$this->rechargeWebhookSecret.$data);
        if (hash_equals($received_digest,$calculated_digest)) {
            return true;
        } else {
            return false;
        }
    }
}
